
BIOMA_WEATHER_COLUMNS =  [ "AtmosphericPressure", "CO2concentration", "WaterTemperatureMinimum", "WaterTemperatureMaximum", "IsEvapotrPresent", "IsVpdPresent", "IsHumidityPresent", "AirRelativeHumidityMinimum", "Windspeed", "VapourPressureDeficit", "SnowDepth", "AirTemperatureMinimum", "AirTemperatureMaximum", "SoilEvaporation", "ReferenceEvapotranspiration", "SurfaceOzoneConcentration", "GlobalSolarRadiation", "IsCO2Present", "Date", "Average_temperature", "Precipitation", "AirRelativeHumidityMaximum", "LeafWetnessDuration", "Grid_no" ]  
BIOMA_LOCATION_COLUMNS = [ "Altitude", "Grid_no", "Latitude", "Longitude", "Distance_to_coast" ]
BIOMA_WEATHER_NAME = 'weather'
BIOMA_LOCATION_NAME = 'location'

BIOMA_SIMPLE_TYPES = ['System.DateTime', 'System.String']